### Appendix S2 for paper: Kasoju et al., "Interspecific variation in number of bristles on forewings of tiny insects does not impact clap-and-fling aerodynamics"

### R code for simulating phylogenies of thrips and fairyflies for phylogenetic comparative analyses
#	(1) Load taxonomic information and make a name list for simulation
#	(2) Define function for simulating the tree from hierarchical taxonomic structure
#	(3) Simulate the trees

### (0) Load libraries
	library(tidyverse) # Version 1.3.0
	library(phytools) # Version 0.7-70

### (1) Load taxonomic information and make a name list for simulation
		name_dat <- read_csv('AppendixS3_all_family_list.csv')
	# Merge each pair of columns, strip redundant values, then 'pull()' to get a list of each level
		name_list <- list()
		name_list[['order']] <- name_dat %>% pull(order) %>% unique()
		name_list[['ord_sub']] <- name_dat %>% unite(x, order, `suborder-superfamily`) %>% pull(x) %>% unique()
		name_list[['sub_fam']] 	<- 	name_dat %>% unite(x, `suborder-superfamily`, family) %>% pull(x) %>% unique()
		name_list[['fam_sub']] 	<- 	name_dat %>% unite(x, family, subfamily) %>% pull(x) %>% unique()
		name_list[['sub_gen']] 	<- 	name_dat %>% unite(x, subfamily, genus) %>% pull(x) %>% unique()

### (2) Define function for simulating the tree from hierarchical taxonomic structure
	tree_sim <- function(name_list, species, ntrees){
		require(phytools)
		# Base tree(s)
		trees <- pbtree(n = length(name_list[[1]]), nsim = ntrees, type = 'continuous')
		# Remove the first level, used to simulate the tree from scratch
		x <- name_list[-1] 
		# For many simulation replicates:
		if(ntrees > 1){
			# Rename tips
			for (i in seq(trees)) trees[[i]]$tip.label <- sample(name_list[[1]]) # In case name_list[[1]] is longer than 2
			# For each level of 'name_list', add tips
			for (j in seq(x)){
				trees <- lapply(trees, genus.to.species.tree, species = x[[j]])
				for(k in seq_along(trees)) trees[[k]]$tip.label <- gsub('\\w+_(\\w+)', '\\1', trees[[k]]$tip.label)
			}
			# Finally, add species you care about (e.g. phenotypic data)
			trees <- lapply(trees, genus.to.species.tree, species = species)
			# Scale to length 1
			for (i in seq(trees)) trees[[i]]$edge.length <- trees[[i]]$edge.length / max(nodeHeights(trees[[i]]))
			# Remove genera unlisted in 'species'
			trees <- lapply(trees, keep.tip, species)
			# Ensure all are ultrametric (i.e. avoid deviations due rounding in floating-point numbers for the branch lengths)
			trees <- lapply(trees, force.ultrametric, method = 'nnls')
		} else {
		# Single-tree operations; the same as above for many
			trees$tip.label <- sample(name_list[[1]])
			for (j in seq(x)){
				trees <- genus.to.species.tree(trees, species = x[[j]])
				trees$tip.label <- gsub('\\w+_(\\w+)', '\\1', trees$tip.label)
			}
			trees <- genus.to.species.tree(trees, species = species)
			trees$edge.length <- trees$edge.length / max(nodeHeights(trees))
			trees <- keep.tip(trees, species)
			trees <- force.ultrametric(trees, method = 'nnls')
		}
		return(trees)
	}		
		
### (3) Simulate the trees
	species <- read_csv('AppendixS1_all_bristle_data.csv') %>%
				pull(species)
	set.seed(2021)
	ntrees <- 10 # Reduce if you are simply testing code and want this to run fast
	trees <- tree_sim(name_list, species, ntrees)
	class(trees) <- 'multiPhylo'
	write.tree(trees, paste0('alltaxa_',ntrees,'random_trees_',Sys.Date(),'.tre'))			
		
		
		
		
		
		
		
		
		
		
		
		
							