### Appendix S5 for paper: Kasoju et al., "Interspecific variation in number of bristles on forewings of tiny insects does not impact clap-and-fling aerodynamics"

### R code for phylogenetic comparative analyses and reproducing associated figures
#	(1) Conduct body-length phylogenetic regressions
#	(2) Plot wing variables on body length, adding the model-averaged regression lines
#	(3) Testing the relationship between wingspan, bristle number, and G/D (PGLS correlation)

### (0) Load preliminary items (libraries, functions, data)
	# (a) Load necessary libraries
		library(tidyverse) # Version 1.3.0
		library(phylolm) # Version 2.6.2

	# (b) Load custom functions for these analyses
		source('AppendixS6_bristled_wings_fxns.R') 
		# Loads functions 'loop_phylolm', 'bl_subsetter', 'rand_phylo_summary', 'looping_phylo_summary',
		# 'bl_subsetter_corr', 'pgls_corr', 'rand_corr_summary', and 'looping_corr_summary'

	# (c) Load data
	#	setwd('~/Dropbox/Research/Vishwa bristles/Data')
		trees <- read.tree('AppendixS4_10000random_trees.tre')
		trees <- trees[1:10] # For working on a subset when testing this code (some analyses are computationally intensive for 10,000 trees)
		dat <- read_csv('AppendixS1_all_bristle_data.csv') %>%
				separate(species, c('genus','sp_epithet'), sep = '_', remove = FALSE) %>%
				select(species, genus, sp_epithet, group:bristle_n, gd)

### (1) Conduct body-length phylogenetic regressions
	# Variables (removed 'wing_area' and 'chord', since Vishwa says their measurement is unreliable)
		response_vars <- c('span', 'bristle_n', 'gd')
	# Loop (note that results are written the current working directory, rather than assigned a distinct object name in the environment)
		for (i in seq(response_vars)){
			# Subset data
				key_vars <- c('BL', response_vars[i])
				x <- bl_subsetter(dat, trees, key_vars)		
			# Run models with lambda MLE
				mod <- 'lambda' # This allows flexibility to run 'phylolm()' with the other evolutionary models it offers
				out <- t(sapply(x[['trees']], function(y) loop_phylolm(x$dat, y, c('log_ind', 'log_dep'), mod)))
				rownames(out) <- paste0('tree',1:nrow(out))
				write.csv(out, file = paste0(response_vars[i],'_',length(trees),'tree_results_',mod,'_',Sys.Date(),'.csv'))	
		}	
	# Summarize analyses (pulls the saved analyses from the previous 'for' loop)
		compiled <- looping_phylo_summary('lambda')
			
### (2) Plot wing variables on body length, adding the model-averaged regression lines
	# First make long-form of data matrix
		long_dat <- dat %>%
					pivot_longer(cols = span:gd, names_to = 'trait') %>% # New column of 'trait' (for different response vars) and 'value' (for the actual value)
					select(-wing_area, -chord, -genus, -sp_epithet) %>%
					mutate(trait = factor(trait, levels = c('gd', 'span', 'bristle_n'))) # To make facets plot in this order, but doesn't seem to work
	# Adjust summary matrix to allow plotting of slopes and intercepts
		slopes <- compiled %>%
					select(trait:thrips_slope_interact_modavg, ends_with('_wgt'), ends_with('_rank'))
		names(slopes) <- gsub('_modavg','',names(slopes))
	# Determine slopes and intercepts of each group			
		slopes <- slopes %>% 					
				mutate(	intercept_Fairyflies = intercept, 
						intercept_Thrips = intercept + thrips_intercept,
						slope_Fairyflies = slope,
						slope_Thrips = slope + thrips_slope_interact
						) %>%
				select(trait, starts_with('intercept_'), starts_with('slope_'), ends_with('_wgt'))
		slopes <- slopes %>%
					pivot_longer(cols = c(-trait, -ends_with('_wgt')), names_to = 'type') %>%
					separate(type, into = c('param', 'group')) %>%
					pivot_wider(names_from = param)
	# Determine bounds of data plotting for lines of each group
		extremes <- dat %>% 
					select(species, group, BL) %>%
					group_by(group) %>%
					summarize(across(where(is_double), list(min = ~min(., na.rm = T), max = ~max(., na.rm = T)), .names = "x_{fn}"))
					# More complicated than necessary b/c I originally summarized many continuous variables
		slopes <- full_join(extremes, slopes)
	# Manually adjust extremes to reflect that the thrips data for bristle_n and span is a 
		# different set of taxa, with different size limits, than gd
		slopes <- slopes %>%
					mutate(	x_min = replace(x_min, group == 'Thrips' & trait == 'gd', 900),
							x_max = replace(x_max, group == 'Thrips' & trait == 'bristle_n', 2940),
							x_max = replace(x_max, group == 'Thrips' & trait == 'span', 2940)
					)		
	# Finish by calculating Y extremes		
		slopes <- slopes %>%
					mutate(
						y_min = intercept + log(x_min) * slope, 
						y_max = intercept + log(x_max) * slope
					) %>% 
					mutate(across(starts_with('y_'), exp)) %>%
					select(x_min, x_max, y_min, y_max, everything()) %>%
					mutate(stat_sig = c('none','groups','interact')[max.col(slopes[,6:8])]) %>% 
					mutate(stat_sig = factor(stat_sig)) %>%
					select(group, trait, stat_sig, everything())
	# Make function to set different breaks and limits per variable:
		breaks_fun <- function(y){
			ind <- .bincode(max(y, na.rm = T), c(1,15,300,2000))
			switch(ind,
				c(2.5, 5, 10),
				c(35,70,140),
				c(250, 500, 1000)
			  )
		}
		limits_fun <- function(y){
			ind <- .bincode(max(y, na.rm = T), c(1,15,300,2000))
			switch(ind,
				c(1 + 2/3, 16 + 2/3),
				c(23 + 1/3, 210),
				c(166 + 2/3, 1666 + 2/3)
			  )
		} # Fxn seems to work fine; unclear why G/D is so problematic
	# Then plot		
		ggplot(long_dat, aes(BL, value, fill = group)) +
			geom_point(na.rm = TRUE, size = 3, shape = 21, color = 'black') +
			scale_x_continuous(	name = 'Body length (micrometers)', 
								trans = 'log', 
								breaks = 175 * c(1,2,4,8,16),
								limits = c(160, 3200),
								expand = c(0,0)
								) + 
			scale_y_continuous(name = NULL,
								trans = 'log',
							#	expand = c(0, 0),
							#	limits = limits_fun, # For some reason this makes G/D lose its axis labels (only; fine for the others)
								breaks = breaks_fun
								) +
			geom_segment(data = slopes, 
						 aes(x = x_min, xend = x_max, y = y_min, yend = y_max, color = group, linetype = stat_sig), 
						 size = 1
						 ) +
			scale_fill_manual(name = 'Taxon', values = c('black', 'gray75')) +
			scale_color_manual(name = 'Taxon', values = c('black', 'gray75')) +
		#	scale_linetype_manual(name = 'Taxon', values = c(22, 'solid')) + # Only two types of models were supported (used in MS for 100,000 simulated trees)
			scale_linetype_manual(name = 'Taxon', values = c(11, 22, 'solid')) + # This line works for three possible statistically supported models (the result when running analyses on the ten trees in line 21)
			guides(linetype = guide_legend(title = 'Stat. significant')) +
			facet_wrap(	~trait, nrow = 2, scales = 'free', 
						as.table = FALSE,
						dir = 'h',
						strip.position = "left", 
						labeller = as_labeller(c(bristle_n = "Number of bristles", gd = "G / D", span = 'Span (micrometers)'))) + # Y on same scale, but this puts a scale next to each facet
			theme_classic() +
			theme(	legend.position = c(1, 1),
					legend.justification = c(1, 1),
					strip.background = element_blank(),
					strip.text = element_text(size = 11),
					strip.placement = 'outside',
					axis.text = element_text(color = 'black'),
					axis.ticks = element_line(color = 'black')
			)
	# Save
		ggsave(paste0('bristle_plots_facetted_',Sys.Date(),'.pdf'), width = 8, height = 6, units = 'in')
		ggsave(paste0('bristle_plots_facetted_',Sys.Date(),'.eps'), width = 8, height = 6, units = 'in')	
	
### (3) Testing the relationship between wingspan, bristle number, and G/D (PGLS correlation, under BM)
	### Loop over all characters
		# Variables examined in the paper
			response_vars <- list(
								c('span', 'bristle_n'),
								c('span', 'gd'),
								c('bristle_n', 'gd')
							)
		# Loop across variable sets
			for (i in seq(response_vars)){
				# Subset data
					key_vars <- response_vars[[i]]
					x <- bl_subsetter_corr(dat, trees, key_vars)		
				# Calculate correlations
					out <- as.data.frame(t(sapply(x[['trees']], function(y) pgls_corr(x$dat, y, cor_type = 'unbiased_r', stat_type = 'z.stat'))))
					rownames(out) <- paste0('tree',1:nrow(out))
					write.csv(out, file = paste0(key_vars[1],'_vs_',key_vars[2],'_',length(trees),'tree_correlations_',Sys.Date(),'.csv'))	
			}	
	### Summarize results
		looping_corr_summary('correlations')		

###	(4) Make new figures of wingspan, bristle number, and G/D on each other (all logged)
	# (a) Plot bristle number vs. wingspan
		ggplot(dat, aes(span, bristle_n, fill = group)) +
			geom_point(na.rm = TRUE, size = 3, shape = 21, color = 'black') +
			scale_x_continuous(	name = 'Wingspan (micrometers)', 
								trans = 'log', 
								breaks = c(175, 250, 350, 500, 700, 1000, 1400),
								limits = c(166, 1666),
								expand = c(0,0)
								) + 
			scale_y_continuous(name = 'Bristle number',
							trans = 'log',
							breaks = c(25, 35, 50, 70, 100, 140, 200),
							limits = c(25, 200), 
							expand = c(0, 0)
							) +
			scale_fill_manual(name = 'Taxon', values = c('black', 'gray75')) +
			scale_color_manual(name = 'Taxon', values = c('black', 'gray75')) +
			theme_classic() +
			theme(	axis.text = element_text(color = 'black'),
					axis.ticks = element_line(color = 'black')
			)
	# Save
		ggsave(paste0('span_vs_bristle_n_',Sys.Date(),'.pdf'), width = 4, height = 3, units = 'in')
		ggsave(paste0('span_vs_bristle_n_',Sys.Date(),'.eps'), width = 4, height = 3, units = 'in')	
	# (b) Plot bristle number vs. G/D
		ggplot(dat, aes(gd, bristle_n)) +
			geom_point(na.rm = TRUE, size = 3, shape = 21, color = 'black', fill = 'black') +
			scale_x_continuous(	name = 'G/D', 
								trans = 'log', 
								breaks = c(3, 4.25, 6, 8.5, 12),
								limits = c(3, 12),
								expand = c(0,0)
								) + 
			scale_y_continuous(name = 'Bristle number',
							trans = 'log',
							breaks = c(25, 35, 50, 70, 100, 140, 200),
							limits = c(25, 200), 
							expand = c(0, 0)
							) +
			theme_classic() +
			theme(	axis.text = element_text(color = 'black'),
					axis.ticks = element_line(color = 'black')
			)
	# Save
		ggsave(paste0('gd_vs_bristle_n_',Sys.Date(),'.pdf'), width = 3, height = 3, units = 'in')
		ggsave(paste0('gd_vs_bristle_n_',Sys.Date(),'.eps'), width = 3, height = 3, units = 'in')	
	# (c) Plot wingspan vs. G/D
		ggplot(dat, aes(gd, span)) +
			geom_point(na.rm = TRUE, size = 3, shape = 21, color = 'black', fill = 'black') +
			scale_x_continuous(	name = 'G/D', 
								trans = 'log', 
								breaks = c(3, 4.25, 6, 8.5, 12),
								limits = c(3, 12),
								expand = c(0,0)
								) + 
			scale_y_continuous(	name = 'Wingspan (micrometers)', 
								trans = 'log', 
								breaks = c(175, 250, 350, 500, 700, 1000, 1400),
								limits = c(166, 1666),
								expand = c(0,0)
								) + 
			theme_classic() +
			theme(	axis.text = element_text(color = 'black'),
					axis.ticks = element_line(color = 'black')
			)
	# Save
		ggsave(paste0('gd_vs_span_',Sys.Date(),'.pdf'), width = 3, height = 3, units = 'in')
		ggsave(paste0('gd_vs_span_',Sys.Date(),'.eps'), width = 3, height = 3, units = 'in')	

	
	
						