### Appendix S6 for paper: Kasoju et al., "Interspecific variation in number of bristles on forewings of tiny insects does not impact clap-and-fling aerodynamics"

### Functions used in other appendices for simulating phylogenies and conducting phylogenetic comparative analyses

### (1) Function to loop 'phylolm()' over trees and extract what I need
	# INPUT:
	#	(1) 'dat' is a dataframe with either rownames 'species' or a column 'species'
	#	(2) 'tree' is a class 'phylo' phylogeny. This is the input/looped variable when running this function in 'lapply()' or 'map()'
	#	(3) 'vars' is a vector of variable names (must be in 'dat'), where the second element is regressed on the first
	#	(4) 'mod' defaults to estimating the MLE of lambda for the residuals
	# OUTPUT:
	#	(1) 'out' is a vector that contains MLEs of the intercept, slope, and lambda (if applicable); estimation variance
	#		of the slope; adjusted R-squared; and the P-value for the fit
		loop_phylolm <- function(dat, tree, vars, mod = 'lambda'){
			# Preliminaries
				require(phylolm)
				if(is_tibble(dat)){ # In case input is class 'tibble'
					dat <- as.data.frame(dat) 
					rownames(dat) <- dat$species
				}
			# Run all four models
				full <- phylolm(formula(paste0(vars[2], '~', 'group *', vars[1])), dat, tree, model = mod) # Slope and intercept differ by 'group'
				noint <- phylolm(formula(paste0(vars[2], '~', 'group +', vars[1])), dat, tree, model = mod) # Same slope, different intercept by 'group'
				nogrp <- phylolm(formula(paste0(vars[2], '~', vars[1])), dat, tree, model = mod) # Same slope and intercept for all levels of 'group'
				null <- phylolm(formula(paste0(vars[2], '~ 1')), dat, tree, model = mod) # Single shared intercept (i.e. no regression line)
			# Prep output (model list, parameter matrix for model-averaging parameter estimates, same for model-averaging slope estimation variance)
				mod_list <- list(null, nogrp, noint, full)
				param_mat <- var_mat <- matrix(nrow = length(mod_list), ncol = length(full$coefficients))
				tmp <- lapply(mod_list, coefficients)			
				param_mat[1,] <- c(tmp[[1]], rep(0,3))
				param_mat[2,] <- c(tmp[[2]][1], 0, tmp[[2]][2], 0)
				param_mat[3,] <- c(tmp[[3]], 0)
				param_mat[4,] <- tmp[[4]]
				tmp <- map(mod_list, ~diag(.$vcov))			
				var_mat[1,] <- c(tmp[[1]], rep(0,3))
				var_mat[2,] <- c(tmp[[2]][1], 0, tmp[[2]][2], 0)
				var_mat[3,] <- c(tmp[[3]], 0)
				var_mat[4,] <- tmp[[4]]				
			# Calculate statistical support for models
				aicc_calc <- function(x) x$aic + (2*x$p^2 + 2*x$p) / (x$n - x$p - 1) # Input 'x' is a list of models
				aicc_wgt_calc <- function(y){ # Input 'y' is a list of AIC scores
					dif <- y - min(y) # Calculate differences between models and best model (i.e. lowest AIC score)
					d <- exp(-0.5 * (dif)) # Exponentiate to produce weights
					wg <- d / sum(d) # Normalize weights
					return(wg) # Return weights
				}
				aicc <- map_dbl(mod_list[1:4], ~aicc_calc(.))
				wgts <- aicc_wgt_calc(aicc)
			# Calculate model-averaged means and variances	
				par_avg <- colSums(param_mat * wgts)				
				var_avg <- colSums(var_mat * wgts)				
			# Finish summaries and naming output
				params <- c('intercept', 'thrips_intercept', 'slope', 'thrips_slope_interact') # These will be model-averaged
				mods <- c('null', 'no_group', 'no_interact', 'full')
				out <- c(par_avg, var_avg)
				out <- c(out, sum(wgts * map_dbl(mod_list, ~.$adj.r.squared))) # Add model-averaged adjusted R-squared
				out <- c(out, aicc, wgts, rank(aicc))
				names(out) <- c(paste0(params, '_modavg'), 
								paste0(params, '_var_modavg'),
								'modavg_adj_R2',
								paste0(mods, '_aicc'),
								paste0(mods, '_wgt'),
								paste0(mods, '_rank')
								)
				if(mod == 'lambda'){
					out[length(out) + 1] <- full$optpar
					names(out)[length(out)] <- 'lambda_MLE'
				}
			# Return results
				return(out)	
		}		
		
### (2) Function for producing subsets of logged data (run within a 'for' loop that specifies variables)
	# INPUT:
	#	(1) 'dat' is a tibble with variables specific to this project (group, species, various continuous variables)
	#	(2) 'trees' is a list of phylogenies over which you will do analyses (assumes multiple trees in a list)
	#	(3) 'key_vars' is a two-element vector with the predictor variable first and the response variable second. They must both be in 'dat'.
	# OUTPUT:
	#	(1) 'out' is a list with two elements: 'dat' is the subset of data with just species, 
	#	group, and key_vars; and a list 'trees' holds the associated phylogenies pruned to include only the data in 'dat'. 
		bl_subsetter <- function(dat, trees, key_vars){
			sub_dat <- dat %>% 
						dplyr::filter(!is.na(get(key_vars[1])), !is.na(get(key_vars[2]))) %>%
						select(species, group, all_of(key_vars)) %>% 
						mutate(log_ind = log(get(key_vars[1])), log_dep = log(get(key_vars[2])))
			sub_sp <- pull(sub_dat, species)
			sub_tr <- lapply(trees, keep.tip, tip = sub_sp)
			out <- list(dat = sub_dat, trees = sub_tr)
			return(out)
		}

### (3) Function to calculate means of regression results across simulated trees, given a matrix of information extracted from each phylogeny
	# Key outputs: means for all parameters estimated, R-squared, AICc, AICc wgt, model ranks, and lambda; SE of scaling slope, averaged across models using AICc weights
	#	 (This SE is calculated as the square root of the sum of slope variance across trees and mean slope estimation variance; see Martins 1996 Evolution 50:12–22)
	# INPUT:
	#	(1) 'x' is a matrix of results from looping the above function 'loop_phylolm' across trees
	# OUTPUT:
	#	(1) 'pars' is a vector of parameter means, plus an estimate of the total slope variance across trees due to tree uncertainty
		rand_phylo_summary <- function(x){
			# Extract means
				lambda <- length(grep('lambda', colnames(x))) # Allows using this function if other models are assumed (i.e. no lambda in output)
				pars <- colMeans(x)
			# Calculate separate variance components, due to estimation error and phylogeny error; from Martins 1996 Evolution
				pars[length(pars) + 1:2] <- c(mean(x[,'slope_var_modavg']), var(x[,'slope_modavg']))
				names(pars)[length(pars) + c(-1,0)] <- c('Vs_slope', 'Vp_slope') # Names from Martins 1996
			# Calculate total variance (sum of that due to estimation error [1st term] and phylogeny error [2nd term]); from Martins 1996 Evolution
				pars[length(pars) + 1] <- sqrt(pars['Vs_slope'] + pars['Vp_slope']) 
				names(pars)[length(pars)] <- 'tot_slope_var'
			# Calculate 95% CI on slope, incorporating both estimation error and phylogeny error; from Martins 1996 Evolution
				pars[length(pars) + 1:2] <- pars['slope_modavg'] + pars['tot_slope_var'] * c(-1.96, 1.96) 
				names(pars)[length(pars) + c(-1,0)] <- c('slope_lower95', 'slope_upper95')
			# Similar to Losos 1994 metric (the proportion of analyses in which the regression is significant): here, the proportion of phylogenies for which the most supported model includes the regression line
				pars[length(pars) + 1] <- mean(max.col(x[,grep('wgt',colnames(x), value = T)]) != 1) # Index '1' corresponds to the intercept-only model (the only one without the regression line)
				names(pars)[length(pars)] <- 'Losos_prop'
			# Kick out
				return(pars)
		}

### (4) Function for looping the summaries across groups (here, 'Thrips' or 'Fairyflies') and
	#	analysis models (i.e. the 'mod' of phylolm(); here, 'BM' or 'lambda'). Writes results
	#	straight to your current working directory in addition to returning an object (if you wish).
		looping_phylo_summary <- function(mod){
			files <- dir(pattern = mod) %>% str_subset('results')
			compiled <- t(sapply(files, function(y) rand_phylo_summary(read.csv(y, row.names = 1))))
			compiled <- as_tibble(compiled)
			compiled$trait <- str_replace(files, '_1.+', '')
			compiled <- select(compiled, trait, everything()) # Reorder to put trait first
			write_csv(compiled, path = paste0('all_',mod,'_summary_',Sys.Date(),'.csv'))
			return(compiled)
		}
		
### (5) Subsetting function to get just the two relevant columns
		# INPUT:
		#	(1) 'dat' is a tibble with variables specific to this project (species, various continuous variables)
		#	(2) 'trees' is a list of phylogenies over which you will do analyses
		#	(3) 'key_vars' is a two-element vector with the predictor variable first and the response variable second. They must both be in 'dat'.
		# OUTPUT:
		#	(1) 'out' is a list with two elements: 'dat' is the subset of data with just species and logged key_vars; 
		#	and a list 'trees' holds the associated phylogenies pruned to include only the data in 'dat'
			bl_subsetter_corr <- function(dat, trees, key_vars){
				sub_dat <- dat %>% 
							dplyr::filter(!is.na(get(key_vars[1])), !is.na(get(key_vars[2]))) %>%
							mutate(log_ind = log(get(key_vars[1])), log_dep = log(get(key_vars[2]))) %>%
							select(species, log_ind, log_dep)
				sub_sp <- pull(sub_dat, species)
				if(class(trees) == 'multiPhylo'){
					sub_tr <- lapply(trees, keep.tip, tip = sub_sp)
				} else {
					sub_tr <- keep.tip(trees, sub_sp)
				}
				out <- list(dat = sub_dat, trees = sub_tr)
				return(out)
			}
					
### (6) PGLS function
		### Input:
		#	(1) 'dat' is a data frame or matrix, with taxa on the rows and 2 traits in the columns
		#	(2) 'tree' is a class 'phylo' phylogeny. Note that the function will prune the tree for the rate calculation
		#	(3) 'lambda' is the ML value from a regression fit of the same variables. Default is full Brownian motion (lambda == 1).
		#		Note that this is not established procedure, but it's included because it has some heuristic uses. 
		#	(4) 'cor_type' is the type of correlation you want: raw correlation or unbiased adjustment for small sample size, following Sokal and Rohlf 1995 Biometry (as discussed in Rohlf 2006)
		#	(5) 'stat_type' is the type of statistic you want to use for the P-value
		#	(6) 'two.tail' is a boolean variable for whether you want a two-tailed (vs. one-tailed) hypothesis test
		### Output:
		#	(1) 'correlation' is the raw correlation (using standard correlation equations)
		#	(2) 'unbiased.cor' is 

			pgls_corr <- function(dat, tree, lambda = 1, cor_type = c('r','unbiased_r'), stat_type = c('r.stat', 'z.stat'), two.tail = TRUE){
				### Function for calculating significance of correlations
				#	'r' is the Pearson product-moment correlation or a PGLS correlation
				#	'n' is your sample size (size of the phylogeny if this is a PGLS correlation)
				#	'type' is how you want to do it: (1) 'r.stat' is based on using r directly (top pg. 575 in Sokal & Rohlf 1995)
				#									 (2) 'z.stat' is based on the z-transformation (mid pg. 575, equation 15.11, and bottom pg. 576, in Sokal & Rohlf 1995)
				#	'two.tail' is logical: do you want a two-tailed test? If FALSE, gives one-tailed probability.
				### Statistical significance can be calculated with standard statistical tables or with this function
				#	Note: The statistical significance of PGLS correlations is calculated like a standard Pearson product-
				#	moment correlation, since the phylogeny is used to weight the correlation estimate (i.e. reducing the
				#	contribution of closely related taxa to the parameter estimate).
					cor.pvalue <- function(r, n, type = c('r.stat', 'z.stat'), two.tail = TRUE){
						if(length(type) == 2) stop("Provide a single value for type of test")
						if(type == 'r.stat'){
							stat <- r * sqrt((n - 2) / (1 - r^2))
							pval <- 1 - pt(stat, n - 2, lower.tail = r > 0)
						} else {
							stat <- atanh(r) * sqrt(n - 3)
							pval <- 1 - pnorm(stat, lower.tail = r > 0)
						}
						if(two.tail) pval <- pval * 2
						out <- setNames(c(stat, pval), c(type, 'P_value'))
						return(out)
					}
				# Now actual calculations
					if(is_tibble(dat)){ # In case input is class 'tibble'
						dat <- as.data.frame(dat) 
						rownames(dat) <- dat$species
						dat$species <- NULL
					}
					d <- as.matrix(dat)
					tr <- tree
					ntax <- length(tr$tip.label)
					nvar <- ncol(d)
					C <- vcv.phylo(tr)
				# Use lambda
					if(lambda != 1){
						for (i in 1:ntax){
							for (j in 1:ntax){
								if (i != j) C[i,j] <- C[i,j] * lambda
							}
						}	
					}
				# Calculate the phylogenetic means (following Rohlf 2006 Evolution, eqn 4)
					ones <- as.matrix(rep(1,ntax))
					anc.states <- ((t(ones) %*% solve(C) %*% ones)^-1) %*%( t(ones) %*% solve(C) %*% d)
				# Calculate covariance matrix, following equation 3 of Rohlf 2006 Evolution
					cov.mat <- (t(d - ones %*% anc.states) %*% solve(C) %*% (d - ones %*% anc.states))/(ntax - 1) 
					colnames(cov.mat) <- row.names(cov.mat) <- colnames(dat)
				# Calculate rate matrix, following standard stats texts
					cor.mat <- cov.mat
					for (i in seq(nvar)){
						for (j in seq(nvar)){
							cor.mat[i,j] <- cov.mat[i,j] / (sqrt(cov.mat[i,i]) * sqrt(cov.mat[j,j]))
						}
					}
				# Get unbiased rates, following eqn. 5 of Rohlf 2006 Evolution
					if(cor_type == 'unbiased_r') cor.mat <- cor.mat * (1 + (1 - cor.mat^2)/(2*(ntax - 4)))
				# Output 1
					out <- setNames(cor.mat[1,2], 'corr')
				# Calculate statistical significance
					stats <- cor.pvalue(out, ntax, type = stat_type, two.tail = two.tail)
					out <- c(out, stats)
				# Kick out results
					return(out)
			}
		
### (7) Function to calculate means of correlations results across simulated trees, given a matrix of results (each row represents a single tree's results)
	# Key outputs: means for correlations, test statistics, P-values, and a single proportion of trees with significant correlations
	#	 (This SE is calculated as the square root of the sum of slope variance across trees and mean slope estimation variance)
	# INPUT:
	#	(1) 'x' is a matrix of results from looping the above function 'loop_phylolm' across trees
	# OUTPUT:
	#	(1) 'pars' is a vector of parameter means, plus an estimate of the total slope variance across trees due to tree uncertainty
		rand_corr_summary <- function(x){
			# Extract means
				pars <- colMeans(x)
			# Losos 1994 metric (the proportion of analyses in which the correlation is significant)
				pars[length(pars) + 1] <- mean(x$P_value < 0.05) 
				names(pars)[length(pars)] <- 'Losos_prop'
			# Kick out
				return(pars)
		}
		
### (8) Function for looping the summaries different results files (e.g. across different combinations of characters)
	#	INPUT: (1) 'search_term' is a key part of the file name you want to search
	#	OUTPUT: None. Writes results straight to your current working directory.
		looping_corr_summary <- function(search_term){
			files <- dir(pattern = search_term)
			compiled <- t(sapply(files, function(y) rand_corr_summary(read.csv(y, row.names = 1))))
			compiled <- as_tibble(compiled)
			compiled$trait <- str_replace(files, '_1.+', '')
			compiled %>% select(trait, everything()) %>% 
						separate(trait, into = paste0('trait',c(1,2)), sep = '_vs_') %>%
						write_csv(path = paste0('all_trait_correlation_summary_',Sys.Date(),'.csv'))				
		}
		
		
		
		
		
		
		
		
		
		
		
		